@@ -24,15 +24,16 @@
 #include <loudmouth/loudmouth.h>
 
 #include <telepathy-glib/base-connection.h>
-#include "gabble-types.h"
+#include "types.h"
 #include "bytestream-iface.h"
 #include "bytestream-ibb.h"
 #include "bytestream-muc.h"
-#include "gabble-connection.h"
+#include "connection.h"
 
 G_BEGIN_DECLS
 
 typedef struct _GabbleBytestreamFactoryClass GabbleBytestreamFactoryClass;
+typedef struct _GabbleBytestreamFactoryPrivate GabbleBytestreamFactoryPrivate;
 
 struct _GabbleBytestreamFactoryClass {
   GObjectClass parent_class;
@@ -41,7 +42,7 @@ struct _GabbleBytestreamFactoryClass {
 struct _GabbleBytestreamFactory {
   GObject parent;
 
-  gpointer priv;
+  GabbleBytestreamFactoryPrivate *priv;
 };
 
 GType gabble_bytestream_factory_get_type (void);
