@@ -280,13 +280,9 @@ gabble_presence_cache_class_init (GabblePresenceCacheClass *klass)
   object_class->set_property = gabble_presence_cache_set_property;
 
   param_spec = g_param_spec_object ("connection", "GabbleConnection object",
-                                    "Gabble connection object that owns this "
-                                    "presence cache.",
-                                    GABBLE_TYPE_CONNECTION,
-                                    G_PARAM_CONSTRUCT_ONLY |
-                                    G_PARAM_READWRITE |
-                                    G_PARAM_STATIC_NICK |
-                                    G_PARAM_STATIC_BLURB);
+      "Gabble connection object that owns this presence cache.",
+      GABBLE_TYPE_CONNECTION,
+      G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
   g_object_class_install_property (object_class,
                                    PROP_CONNECTION,
                                    param_spec);
@@ -839,7 +835,7 @@ _caps_disco_cb (GabbleDisco *disco,
    * stanza. */
   if (!tp_strdiff (waiter_self->hash, "sha-1"))
     {
-      const gchar *computed_hash;
+      gchar *computed_hash;
       trust_inc = CAPABILITY_BUNDLE_ENOUGH_TRUST;
 
       computed_hash = caps_hash_compute_from_lm_node (query_result);
@@ -856,6 +852,8 @@ _caps_disco_cb (GabbleDisco *disco,
           trust = 0;
           bad_hash = TRUE;
         }
+
+      g_free (computed_hash);
     }
   else
     {
@@ -1475,3 +1473,82 @@ gabble_presence_cache_really_remove (
   tp_handle_set_remove (priv->presence_handles, handle);
 }
 
+void
+gabble_presence_cache_contacts_added_to_olpc_view (GabblePresenceCache *self,
+                                                   TpHandleSet *handles)
+{
+  GArray *tmp, *changed;
+  guint i;
+
+  tmp = tp_handle_set_to_array (handles);
+
+  changed = g_array_new (FALSE, FALSE, sizeof (TpHandle));
+
+  for (i = 0; i < tmp->len; i++)
+    {
+      TpHandle handle;
+      GabblePresence *presence;
+
+      handle = g_array_index (tmp, TpHandle, i);
+
+      presence = gabble_presence_cache_get (self, handle);
+      if (presence == NULL)
+        {
+          presence = _cache_insert (self, handle);
+        }
+
+      if (gabble_presence_added_to_view (presence))
+        {
+          g_array_append_val (changed, handle);
+        }
+    }
+
+  if (changed->len > 0)
+    {
+      g_signal_emit (self, signals[PRESENCES_UPDATED], 0, changed);
+    }
+
+  g_array_free (tmp, TRUE);
+  g_array_free (changed, TRUE);
+}
+
+void
+gabble_presence_cache_contacts_removed_from_olpc_view (
+    GabblePresenceCache *self,
+    TpHandleSet *handles)
+{
+  GArray *tmp, *changed;
+  guint i;
+
+  tmp = tp_handle_set_to_array (handles);
+
+  changed = g_array_new (FALSE, FALSE, sizeof (TpHandle));
+
+  for (i = 0; i < tmp->len; i++)
+    {
+      TpHandle handle;
+      GabblePresence *presence;
+
+      handle = g_array_index (tmp, TpHandle, i);
+
+      presence = gabble_presence_cache_get (self, handle);
+      if (presence == NULL)
+        {
+          presence = _cache_insert (self, handle);
+        }
+
+      if (gabble_presence_removed_from_view (presence))
+        {
+          g_array_append_val (changed, handle);
+          gabble_presence_cache_maybe_remove (self, handle);
+        }
+    }
+
+  if (changed->len > 0)
+    {
+      g_signal_emit (self, signals[PRESENCES_UPDATED], 0, changed);
+    }
+
+  g_array_free (tmp, TRUE);
+  g_array_free (changed, TRUE);
+}
