@@ -105,8 +105,7 @@ gabble_request_pipeline_class_init (GabbleRequestPipelineClass *cls)
   param_spec = g_param_spec_object ("connection", "GabbleConnection object",
       "Gabble connection object that owns this request pipeline helper "
       "object.", GABBLE_TYPE_CONNECTION,
-      G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE | G_PARAM_STATIC_NICK |
-      G_PARAM_STATIC_BLURB);
+      G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
   g_object_class_install_property (object_class, PROP_CONNECTION, param_spec);
 
 }
