@@ -20,7 +20,10 @@
 #include "config.h"
 #include "tube-iface.h"
 
+#include <telepathy-glib/gtypes.h>
+
 #include "connection.h"
+#include "util.h"
 
 gboolean
 gabble_tube_iface_accept (GabbleTubeIface *self,
@@ -33,12 +36,12 @@ gabble_tube_iface_accept (GabbleTubeIface *self,
 }
 
 void
-gabble_tube_iface_close (GabbleTubeIface *self)
+gabble_tube_iface_close (GabbleTubeIface *self, gboolean closed_remotely)
 {
-  void (*virtual_method)(GabbleTubeIface *) =
+  void (*virtual_method)(GabbleTubeIface *, gboolean) =
     GABBLE_TUBE_IFACE_GET_CLASS (self)->close;
   g_assert (virtual_method != NULL);
-  virtual_method (self);
+  virtual_method (self, closed_remotely);
 }
 
 void
@@ -134,18 +137,6 @@ gabble_tube_iface_base_init (gpointer klass)
           G_PARAM_STATIC_BLURB);
       g_object_interface_install_property (klass, param_spec);
 
-      param_spec = g_param_spec_uint (
-          "initiator",
-          "Initiator handle",
-          "The TpHandle of the initiator of this tube object.",
-          0, G_MAXUINT32, 0,
-          G_PARAM_CONSTRUCT_ONLY |
-          G_PARAM_READWRITE |
-          G_PARAM_STATIC_NAME |
-          G_PARAM_STATIC_NICK |
-          G_PARAM_STATIC_BLURB);
-      g_object_interface_install_property (klass, param_spec);
-
       param_spec = g_param_spec_string (
           "service",
           "service name",
@@ -162,8 +153,7 @@ gabble_tube_iface_base_init (gpointer klass)
           "parameters",
           "parameters GHashTable",
           "GHashTable containing parameters of this tube object.",
-          G_TYPE_HASH_TABLE,
-          G_PARAM_CONSTRUCT_ONLY |
+          TP_HASH_TYPE_STRING_VARIANT_MAP,
           G_PARAM_READWRITE |
           G_PARAM_STATIC_NAME |
           G_PARAM_STATIC_NICK |
@@ -209,3 +199,80 @@ gabble_tube_iface_get_type (void)
 
   return type;
 }
+
+
+void
+gabble_tube_iface_publish_in_node (GabbleTubeIface *tube,
+                                   TpBaseConnection *conn,
+                                   LmMessageNode *node)
+{
+  LmMessageNode *parameters_node;
+  GHashTable *parameters;
+  TpTubeType type;
+  gchar *service, *id_str;
+  guint tube_id;
+  TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (
+    conn, TP_HANDLE_TYPE_CONTACT);
+  TpHandle initiator_handle;
+
+  g_object_get (G_OBJECT (tube),
+      "type", &type,
+      "initiator-handle", &initiator_handle,
+      "service", &service,
+      "parameters", &parameters,
+      "id", &tube_id,
+      NULL);
+
+  id_str = g_strdup_printf ("%u", tube_id);
+
+  lm_message_node_set_attributes (node,
+      "service", service,
+      "id", id_str,
+      NULL);
+
+  g_free (id_str);
+
+  switch (type)
+    {
+      case TP_TUBE_TYPE_DBUS:
+        {
+          gchar *name, *stream_id;
+
+          g_object_get (G_OBJECT (tube),
+              "stream-id", &stream_id,
+              "dbus-name", &name,
+              NULL);
+
+          lm_message_node_set_attributes (node,
+              "type", "dbus",
+              "stream-id", stream_id,
+              "initiator", tp_handle_inspect (contact_repo, initiator_handle),
+              NULL);
+
+          if (name != NULL)
+            lm_message_node_set_attribute (node, "dbus-name", name);
+
+          g_free (name);
+          g_free (stream_id);
+        }
+        break;
+      case TP_TUBE_TYPE_STREAM:
+        {
+          lm_message_node_set_attribute (node, "type", "stream");
+        }
+        break;
+      default:
+        {
+          g_return_if_reached ();
+        }
+    }
+
+  parameters_node = lm_message_node_add_child (node, "parameters",
+      NULL);
+  lm_message_node_add_children_from_properties (parameters_node, parameters,
+      "parameter");
+
+  g_free (service);
+  g_hash_table_unref (parameters);
+}
+
