@@ -18,13 +18,15 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
+#include "config.h"
+
 #include <stdio.h>
 
 #include <dbus/dbus-glib.h>
 #include <dbus/dbus-protocol.h>
-
 #include <telepathy-glib/enums.h>
-#include "gabble-connection-manager.h"
+
+#include "connection-manager.h"
 
 static gchar *
 mgr_file_contents (const char *busname,
