@@ -1,7 +1,9 @@
 
+#include "config.h"
+
 #include <string.h>
 
-#include "text-mixin.h"
+#include "src/text-mixin.h"
 
 /* Test the most basic <message> possible. */
 static gboolean
