@@ -8,6 +8,7 @@ from twisted.words.xish import domish, xpath
 
 from gabbletest import go, make_result_iq, exec_test, acknowledge_iq
 from servicetest import call_async, EventPattern
+import ns
 
 def test(q, bus, conn, stream):
     conn.Connect()
@@ -28,7 +29,7 @@ def test(q, bus, conn, stream):
     message['from'] = 'bob@localhost'
     message['to'] = 'test@localhost'
     properties = message.addElement(
-        ('http://laptop.org/xmpp/activity-properties', 'properties'))
+        (ns.OLPC_ACTIVITY_PROPS, 'properties'))
     properties['room'] = 'chat@conf.localhost'
     properties['activity'] = 'foo_id'
     property = properties.addElement((None, 'property'))
@@ -45,7 +46,7 @@ def test(q, bus, conn, stream):
     message = domish.Element((None, 'message'))
     message['from'] = 'chat@conf.localhost'
     message['to'] = 'test@localhost'
-    x = message.addElement(('http://jabber.org/protocol/muc#user', 'x'))
+    x = message.addElement((ns.MUC_USER, 'x'))
     invite = x.addElement((None, 'invite'))
     invite['from'] = 'bob@localhost'
     reason = invite.addElement((None, 'reason'))
@@ -106,7 +107,7 @@ def test(q, bus, conn, stream):
     message['from'] = 'bob@localhost'
     message['to'] = 'test@localhost'
     properties = message.addElement(
-        ('http://laptop.org/xmpp/activity-properties', 'properties'))
+        (ns.OLPC_ACTIVITY_PROPS, 'properties'))
     properties['room'] = 'chat@conf.localhost'
     properties['activity'] = 'foo_id'
     property = properties.addElement((None, 'property'))
@@ -140,7 +141,7 @@ def test(q, bus, conn, stream):
     # Send presence for own membership of room.
     presence = domish.Element((None, 'presence'))
     presence['from'] = 'chat@conf.localhost/test'
-    x = presence.addElement(('http://jabber.org/protocol/muc#user', 'x'))
+    x = presence.addElement((ns.MUC_USER, 'x'))
     item = x.addElement('item')
     item['affiliation'] = 'owner'
     item['role'] = 'moderator'
@@ -167,7 +168,7 @@ def test(q, bus, conn, stream):
     message['from'] = 'chat@conf.localhost/bob'
     message['to'] = 'chat@conf.localhost'
     properties = message.addElement(
-        ('http://laptop.org/xmpp/activity-properties', 'properties'))
+        (ns.OLPC_ACTIVITY_PROPS, 'properties'))
     properties['activity'] = 'foo_id'
     property = properties.addElement((None, 'property'))
     property['type'] = 'str'
@@ -186,11 +187,11 @@ def test(q, bus, conn, stream):
     activities = xpath.queryForNodes('/iq/pubsub/publish/item/activities',
             message)
     assert (activities is not None and len(activities) == 1), repr(activities)
-    assert activities[0].uri == 'http://laptop.org/xmpp/activity-properties'
+    assert activities[0].uri == ns.OLPC_ACTIVITY_PROPS
 
     properties = xpath.queryForNodes('/activities/properties', activities[0])
     assert (properties is not None and len(properties) == 1), repr(properties)
-    assert properties[0].uri == 'http://laptop.org/xmpp/activity-properties'
+    assert properties[0].uri == ns.OLPC_ACTIVITY_PROPS
     assert properties[0]['room'] == 'chat@conf.localhost'
     assert properties[0]['activity'] == 'foo_id'
 
@@ -226,7 +227,7 @@ def test(q, bus, conn, stream):
 
     properties = xpath.queryForNodes('/message/properties', message)
     assert (properties is not None and len(properties) == 1), repr(properties)
-    assert properties[0].uri == 'http://laptop.org/xmpp/activity-properties'
+    assert properties[0].uri == ns.OLPC_ACTIVITY_PROPS
     assert properties[0]['room'] == 'chat@conf.localhost'
     assert properties[0]['activity'] == 'foo_id'
 
@@ -258,7 +259,7 @@ def test(q, bus, conn, stream):
     activities = xpath.queryForNodes('/iq/pubsub/publish/item/activities',
             message)
     assert (activities is not None and len(activities) == 1), repr(activities)
-    assert activities[0].uri == 'http://laptop.org/xmpp/activity-properties'
+    assert activities[0].uri == ns.OLPC_ACTIVITY_PROPS
 
     properties = xpath.queryForNodes('/activities/properties', activities[0])
     assert properties is None, repr(properties)
@@ -274,7 +275,7 @@ def test(q, bus, conn, stream):
     activities = xpath.queryForNodes('/iq/pubsub/publish/item/activities',
             message)
     assert (activities is not None and len(activities) == 1), repr(activities)
-    assert activities[0].uri == 'http://laptop.org/xmpp/activities'
+    assert activities[0].uri == ns.OLPC_ACTIVITIES
 
     activity = xpath.queryForNodes('/activities/activity', activities[0])
     assert activity is None, repr(activity)
@@ -291,7 +292,7 @@ def test(q, bus, conn, stream):
 
     properties = xpath.queryForNodes('/message/properties', message)
     assert (properties is not None and len(properties) == 1), repr(properties)
-    assert properties[0].uri == 'http://laptop.org/xmpp/activity-properties'
+    assert properties[0].uri == ns.OLPC_ACTIVITY_PROPS
     assert properties[0]['room'] == 'chat@conf.localhost'
     assert properties[0]['activity'] == 'foo_id'
 
@@ -322,11 +323,11 @@ def test(q, bus, conn, stream):
     activities = xpath.queryForNodes('/iq/pubsub/publish/item/activities',
             message)
     assert (activities is not None and len(activities) == 1), repr(activities)
-    assert activities[0].uri == 'http://laptop.org/xmpp/activity-properties'
+    assert activities[0].uri == ns.OLPC_ACTIVITY_PROPS
 
     properties = xpath.queryForNodes('/activities/properties', activities[0])
     assert (properties is not None and len(properties) == 1), repr(properties)
-    assert properties[0].uri == 'http://laptop.org/xmpp/activity-properties'
+    assert properties[0].uri == ns.OLPC_ACTIVITY_PROPS
     assert properties[0]['room'] == 'chat@conf.localhost'
     assert properties[0]['activity'] == 'foo_id'
 
@@ -357,7 +358,7 @@ def test(q, bus, conn, stream):
     activities = xpath.queryForNodes('/iq/pubsub/publish/item/activities',
             message)
     assert (activities is not None and len(activities) == 1), repr(activities)
-    assert activities[0].uri == 'http://laptop.org/xmpp/activities'
+    assert activities[0].uri == ns.OLPC_ACTIVITIES
 
     activity = xpath.queryForNodes('/activities/activity', activities[0])
     assert (activity is not None and len(activity) == 1), repr(activity)
@@ -379,7 +380,7 @@ def test(q, bus, conn, stream):
     activities = xpath.queryForNodes('/iq/pubsub/publish/item/activities',
             message)
     assert (activities is not None and len(activities) == 1), repr(activities)
-    assert activities[0].uri == 'http://laptop.org/xmpp/activities'
+    assert activities[0].uri == ns.OLPC_ACTIVITIES
 
     activity = xpath.queryForNodes('/activities/activity', activities[0])
     assert activity is None, repr(activity)
@@ -395,7 +396,7 @@ def test(q, bus, conn, stream):
     activities = xpath.queryForNodes('/iq/pubsub/publish/item/activities',
             message)
     assert (activities is not None and len(activities) == 1), repr(activities)
-    assert activities[0].uri == 'http://laptop.org/xmpp/activity-properties'
+    assert activities[0].uri == ns.OLPC_ACTIVITY_PROPS
 
     properties = xpath.queryForNodes('/activities/properties', activities[0])
     assert properties is None, repr(properties)
